@@ -23,10 +23,10 @@ import logging
 from telnetlib import Telnet
 from time import time, sleep
 
-from message import Command, Message
+from message import Command, MessageFactory
 
 class TeamspeakConnection(Telnet):
-    def __init__(self, hostname, port, timeout, pipe_in, pipe_out, keep_alive=30, poll_interval=0.25):
+    def __init__(self, hostname, port, timeout, pipe_in, pipe_out, keep_alive=30, poll_interval=0.125):
         self.pipe_in = pipe_in
         self.pipe_out = pipe_out
 
@@ -41,7 +41,7 @@ class TeamspeakConnection(Telnet):
     def write_command(self, command):
         self.logger.info("Sending command %s" % command.__repr__())
         self.commands_unresponded.append(command)
-        self.write("%s\n" % str(command))
+        self.write("%s\n" % command.output)
 
     def write_keep_alive(self):
         self.logger.debug("Sending keepalive message.")
@@ -49,30 +49,44 @@ class TeamspeakConnection(Telnet):
 
     def main_loop(self):
         while True:
-            if not self.pipe_in.empty():
-                comm = self.pipe_in.get_nowait()
-                if isinstance(comm, Command):
-                    self.write_command(comm)
             incoming = self.receive_message()
             if incoming:
                 self.pipe_out.put(incoming)
-            if int(time()) % self.keep_alive == 0:
-                self.write_keep_alive()
+            else:
+                # Only write messages if we have nothing incoming
+                if not self.pipe_in.empty():
+                    comm = self.pipe_in.get_nowait()
+                    if isinstance(comm, Command):
+                        self.write_command(comm)
+                elif int(time()) % self.keep_alive == 0:
+                    self.write_keep_alive()
             sleep(self.poll_interval)
 
     def receive_message(self):
         try:
-            incoming_message = self.read_until('\n', self.timeout)
-            message = Message(incoming_message)
-            if message.is_response():
-                message.set_origination(
+            incoming_message = self.read_until('\n', self.timeout).strip()
+            if incoming_message.strip():
+                self.logger.debug("Incoming string \"%s\"" % incoming_message)
+                message = MessageFactory.get_message(incoming_message)
+                if message:
+                    if message.is_response():
+                        message.set_origination(
+                                    self.commands_unresponded.popleft()
+                                )
+                    elif message.is_reset_message():
+                        # Command didn't ask for a response
+                        if self.commands_unresponded:
                             self.commands_unresponded.popleft()
-                        )
-            if message.is_reset_message():
-                self.commands_unresponded = deque()
-            self.logger.info("Received message %s" % message.__repr__())
-            return message
-        except ValueError:
-            return None
+                    self.logger.info("Received message %s" % message.__repr__())
+                    return message
+        except ValueError as e:
+            pass
+        except IndexError as e:
+            self.logger.warning(
+                    "Unable to create message for \"%s\"; %s" % (
+                        incoming_message, 
+                        e
+                    )
+                )
         except Exception as e:
             return e
